(in-package :LED-cube)

(defparameter *transformations* nil)

(defmacro define-cube-transformation (name lambda-list &rest body)
  `(progn
     (pushnew ',name *transformations*)
     (defun ,name ,lambda-list
     ,@body)))

(define-cube-transformation copy-x-0-0-to-all ()
  (save-undo)
  (dolist (cube *animation*)
    (dotimes (x 5)
      (let ((led (aref cube x 0 0)))
;;	(princ led)
	(dotimes (y 5)
	  (dotimes (z 5)
	    (setf (aref cube x y z) led))))
;;      (princ #\newline)
      )))

(define-cube-transformation mirrored-animation ()
  (save-undo)
  (setf *animation* (nreverse (mapcar #'copy-cube *animation*))))

(define-cube-transformation append-mirrored-animation ()
  (save-undo)
  (setf *animation* (append *animation* (nreverse (mapcar #'copy-cube *animation*)))))

(defun invert-LEDs (&optional (cube (current-cube)))
  (let ((new-cube (make-cube)))
    (do-cube-times (x y z)
      (setf (aref new-cube x y z)
            (ensure-intensity-to-be-of-character
             (- 5 (ensure-intensity-to-be-of-number (aref cube x y z))))))
    new-cube))

(define-cube-transformation invert-current-LEDs ()
  (save-undo)
  (setf (current-cube) (invert-LEDs)))

(define-cube-transformation invert-animation ()
  (save-undo)
  (setf *animation* (mapcar (lambda (cube)
                              (invert-LEDs (copy-cube cube)))
                            *animation*)))

(define-cube-transformation append-inverted-animation ()
  (save-undo)
  (setf *animation* (append *animation*
                            (mapcar (lambda (cube)
                                      (invert-LEDs (copy-cube cube)))
                                       *animation*))))
